<?php
/**
 * Test the resource handling.
 *
 * PHP version 5
 *
 * @category   Kolab
 * @package    Kolab_Resource
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_Resource
 */

/**
 * Prepare the test setup.
 */
require_once dirname(__FILE__) . '/../Autoload.php';

/**
 * Test the resource handling.
 *
 * Copyright 2008-2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category   Kolab
 * @package    Kolab_Resource
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_Resource
 */
class Horde_Kolab_Resource_Integration_ResourceTest
extends Horde_Kolab_Resource_TestCase
{
    public function setUp()
    {
        global $conf;

        require_once 'Horde/Kolab/Server.php';

        $_SERVER['SERVER_NAME'] = 'home.example.org';

        $GLOBALS['KOLAB_SERVER_TEST_DATA'] = array();

        /** Prepare a Kolab test server */
        $conf['kolab']['server']['driver'] = 'test';

        $this->server = Horde_Kolab_Server::singleton();

        /** Ensure we don't use a connection from older tests */
        $this->server->unbind();

        /** Set base DN */
        $this->server->_base_dn = 'dc=example,dc=org';

        $this->server->add(
            array('givenName' => 'Gunnar',
                  'sn' => 'Wrobel',
                  'type' => KOLAB_OBJECT_USER,
                  'mail' => 'wrobel@example.org',
                  'uid' => 'wrobel',
                  'userPassword' => 'none',
                  'kolabHomeServer' => 'home.example.org',
                  'kolabImapServer' => 'imap.example.org',
                  'kolabFreeBusyServer' => 'https://fb.example.org/freebusy',
                  KOLAB_ATTR_IPOLICY => array('ACT_MANUAL'),
                  'alias' => array('gunnar@example.org',
                                   'g.wrobel@example.org'),
            )
        );

        $this->server->add(
            array('givenName' => 'Gunnar2',
                  'sn' => 'Wrobel2',
                  'type' => KOLAB_OBJECT_USER,
                  'mail' => 'wrobel2@example.org',
                  'uid' => 'wrobel2',
                  'userPassword' => 'none',
                  'kolabHomeServer' => 'home.example.org',
                  'kolabImapServer' => 'imap.example.org',
                  'kolabFreeBusyServer' => 'https://fb.example.org/freebusy',
                  KOLAB_ATTR_IPOLICY => array('ACT_REJECT_IF_CONFLICT'),
                  'alias' => array('gunnar2@example.org',
                                   'g.wrobel2@example.org'),
            )
        );

        $this->server->add(
            array('cn' => 'calendar',
                  'sn' => 'calendar',
                  'givenName' => '',
                  'type' => KOLAB_OBJECT_USER,
                  'mail' => 'calendar@example.org',
                  'uid' => 'calendar@home.example.org',
                  'userPassword' => 'calendar',
                  'kolabHomeServer' => 'home.example.org',
                  'kolabImapServer' => 'imap.example.org',
            )
        );

        require_once 'Horde/Kolab/Storage/List.php';

        /** Ensure that IMAP runs in testing mode and is empty */
        $GLOBALS['KOLAB_TESTING'] = array();

        /** Prepare a Kolab test storage */
        $this->storage = Kolab_List::singleton(true);

        $conf['kolab']['no_triggering'] = true;

        $conf['kolab']['filter']['reject_forged_from_header'] = false;
        $conf['kolab']['filter']['email_domain'] = 'example.org';
        $conf['kolab']['filter']['privileged_networks'] = '127.0.0.1,192.168.0.0/16';
        $conf['kolab']['filter']['verify_from_header'] = true;
        $conf['kolab']['filter']['calendar_id'] = 'calendar';
        $conf['kolab']['filter']['calendar_pass'] = 'calendar';
        $conf['kolab']['filter']['calendar_store'] = 'Calendar';
        $conf['kolab']['filter']['lmtp_host'] = 'imap.example.org';
        $conf['kolab']['filter']['simple_locks'] = true;
        $conf['kolab']['filter']['simple_locks_timeout'] = 3;
    }

    /**
     * Test return value for a non-local resource
     */
    public function testUserOnDifferentHomeServer()
    {
        $r = new Kolab_Resource();
        $this->assertTrue(
            $r->handleMessage(
                'localhost',
                'test@example.org',
                'wrobel@example.org',
                null
            )
        );
    }

    /**
     * Test manual actions
     */
    public function testManualAction()
    {
        require_once 'Horde.php';
        $r = new Kolab_Resource();
        $this->assertTrue(
            $r->handleMessage(
                'home.example.org',
                'test@example.org',
                'wrobel@example.org',
                dirname(__FILE__) . '/../fixtures/recur_invitation.eml'
            )
        );
    }

    /**
     * Test invitation.
     */
    public function testRecurrenceInvitation()
    {
        require_once 'Horde/iCalendar.php';
        require_once 'Horde/iCalendar/vfreebusy.php';

        $GLOBALS['KOLAB_FILTER_TESTING'] = new Horde_iCalendar_vfreebusy();
        $GLOBALS['KOLAB_FILTER_TESTING']->setAttribute('DTSTART', Horde_iCalendar::_parseDateTime('20080926T000000Z'));
        $GLOBALS['KOLAB_FILTER_TESTING']->setAttribute('DTEND', Horde_iCalendar::_parseDateTime('20081126T000000Z'));

        include_once 'Horde.php';
        $r = new Kolab_Resource();
        $response = $r->handleMessage(
            'home.example.org',
            'test@example.org',
            'wrobel2@example.org',
            dirname(__FILE__) . '/../fixtures/recur_invitation.eml'
        );
        $r->cleanup();
        $data = $response->getData();
        $this->assertContains('SUMMARY:trg', $data);
        $this->assertContains('Subject: Accepted: trg', $data);
        $this->assertContains('DESCRIPTION:Zeigen Sie Ihren Termin unter', $data);
        $this->assertContains('LOCATION:Somewhere', $data);
    }

    /**
     * Test invitation.
     */
    public function testOutsideBookablePeriod()
    {
        require_once 'Horde/iCalendar.php';
        require_once 'Horde/iCalendar/vfreebusy.php';

        $GLOBALS['KOLAB_FILTER_TESTING'] = new Horde_iCalendar_vfreebusy();
        $GLOBALS['KOLAB_FILTER_TESTING']->setAttribute('DTSTART', Horde_iCalendar::_parseDateTime('20060926T000000Z'));
        $GLOBALS['KOLAB_FILTER_TESTING']->setAttribute('DTEND', Horde_iCalendar::_parseDateTime('20061126T000000Z'));

        include_once 'Horde.php';
        $r = new Kolab_Resource();
        $response = $r->handleMessage(
            'home.example.org',
            'test@example.org',
            'wrobel2@example.org',
            dirname(__FILE__) . '/../fixtures/recur_invitation.eml'
        );
        $r->cleanup();
        $data = $response->getData();
        $this->assertContains('Subject: Declined [outside bookable period]: trg', $data);
    }
}
